/****************************************************************************
** Meta object code from reading C++ file 'qoauth2deviceauthorizationflow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtnetworkauth/src/oauth/qoauth2deviceauthorizationflow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qoauth2deviceauthorizationflow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t {};
} // unnamed namespace

template <> constexpr inline auto QOAuth2DeviceAuthorizationFlow::qt_create_metaobjectdata<qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QOAuth2DeviceAuthorizationFlow",
        "authorizeWithUserCode",
        "",
        "QUrl",
        "verificationUrl",
        "userCode",
        "completeVerificationUrl",
        "userCodeChanged",
        "verificationUrlChanged",
        "completeVerificationUrlChanged",
        "pollingChanged",
        "polling",
        "userCodeExpirationAtChanged",
        "expiration",
        "grant",
        "startTokenPolling",
        "stopTokenPolling",
        "refreshTokensImplementation",
        "userCodeExpirationAt"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'authorizeWithUserCode'
        QtMocHelpers::SignalData<void(const QUrl &, const QString &, const QUrl &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { QMetaType::QString, 5 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'userCodeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'verificationUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'completeVerificationUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 6 },
        }}),
        // Signal 'pollingChanged'
        QtMocHelpers::SignalData<void(bool)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 11 },
        }}),
        // Signal 'userCodeExpirationAtChanged'
        QtMocHelpers::SignalData<void(const QDateTime &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 13 },
        }}),
        // Slot 'grant'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'startTokenPolling'
        QtMocHelpers::SlotData<bool()>(15, 2, QMC::AccessPublic, QMetaType::Bool),
        // Slot 'stopTokenPolling'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refreshTokensImplementation'
        QtMocHelpers::SlotData<void()>(17, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'userCode'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Final, 1),
        // property 'verificationUrl'
        QtMocHelpers::PropertyData<QUrl>(4, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 2),
        // property 'completeVerificationUrl'
        QtMocHelpers::PropertyData<QUrl>(6, 0x80000000 | 3, QMC::DefaultPropertyFlags | QMC::EnumOrFlag | QMC::Final, 3),
        // property 'polling'
        QtMocHelpers::PropertyData<bool>(11, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Final, 4),
        // property 'userCodeExpirationAt'
        QtMocHelpers::PropertyData<QDateTime>(18, QMetaType::QDateTime, QMC::DefaultPropertyFlags | QMC::Final, 5),
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<QOAuth2DeviceAuthorizationFlow, qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QOAuth2DeviceAuthorizationFlow::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractOAuth2::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>.metaTypes,
    nullptr
} };

void QOAuth2DeviceAuthorizationFlow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QOAuth2DeviceAuthorizationFlow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->authorizeWithUserCode((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 1: _t->userCodeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->verificationUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 3: _t->completeVerificationUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->pollingChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 5: _t->userCodeExpirationAtChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 6: _t->grant(); break;
        case 7: { bool _r = _t->startTokenPolling();
            if (_a[0]) *reinterpret_cast<bool*>(_a[0]) = std::move(_r); }  break;
        case 8: _t->stopTokenPolling(); break;
        case 9: _t->refreshTokensImplementation(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(const QUrl & , const QString & , const QUrl & )>(_a, &QOAuth2DeviceAuthorizationFlow::authorizeWithUserCode, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(const QString & )>(_a, &QOAuth2DeviceAuthorizationFlow::userCodeChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(const QUrl & )>(_a, &QOAuth2DeviceAuthorizationFlow::verificationUrlChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(const QUrl & )>(_a, &QOAuth2DeviceAuthorizationFlow::completeVerificationUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(bool )>(_a, &QOAuth2DeviceAuthorizationFlow::pollingChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth2DeviceAuthorizationFlow::*)(const QDateTime & )>(_a, &QOAuth2DeviceAuthorizationFlow::userCodeExpirationAtChanged, 5))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->userCode(); break;
        case 1: *reinterpret_cast<QUrl*>(_v) = _t->verificationUrl(); break;
        case 2: *reinterpret_cast<QUrl*>(_v) = _t->completeVerificationUrl(); break;
        case 3: *reinterpret_cast<bool*>(_v) = _t->isPolling(); break;
        case 4: *reinterpret_cast<QDateTime*>(_v) = _t->userCodeExpirationAt(); break;
        default: break;
        }
    }
}

const QMetaObject *QOAuth2DeviceAuthorizationFlow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QOAuth2DeviceAuthorizationFlow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN30QOAuth2DeviceAuthorizationFlowE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractOAuth2::qt_metacast(_clname);
}

int QOAuth2DeviceAuthorizationFlow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractOAuth2::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 5;
    }
    return _id;
}

// SIGNAL 0
void QOAuth2DeviceAuthorizationFlow::authorizeWithUserCode(const QUrl & _t1, const QString & _t2, const QUrl & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2, _t3);
}

// SIGNAL 1
void QOAuth2DeviceAuthorizationFlow::userCodeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QOAuth2DeviceAuthorizationFlow::verificationUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QOAuth2DeviceAuthorizationFlow::completeVerificationUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QOAuth2DeviceAuthorizationFlow::pollingChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QOAuth2DeviceAuthorizationFlow::userCodeExpirationAtChanged(const QDateTime & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}
QT_WARNING_POP
