/****************************************************************************
** Meta object code from reading C++ file 'qoauth2authorizationcodeflow.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtnetworkauth/src/oauth/qoauth2authorizationcodeflow.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qoauth2authorizationcodeflow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t {};
} // unnamed namespace

template <> constexpr inline auto QOAuth2AuthorizationCodeFlow::qt_create_metaobjectdata<qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QOAuth2AuthorizationCodeFlow",
        "RegisterEnumClassesUnscoped",
        "false",
        "accessTokenUrlChanged",
        "",
        "QUrl",
        "accessTokenUrl",
        "grant",
        "refreshAccessToken",
        "refreshTokensImplementation",
        "PkceMethod",
        "S256",
        "Plain",
        "None"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'accessTokenUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(3, 4, QMC::AccessPublic | QMC::MethodCompatibility, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Slot 'grant'
        QtMocHelpers::SlotData<void()>(7, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refreshAccessToken'
        QtMocHelpers::SlotData<void()>(8, 4, QMC::AccessPublic, QMetaType::Void),
        // Slot 'refreshTokensImplementation'
        QtMocHelpers::SlotData<void()>(9, 4, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'accessTokenUrl'
        QtMocHelpers::PropertyData<QUrl>(6, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 0),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'PkceMethod'
        QtMocHelpers::EnumData<enum PkceMethod>(10, 10, QMC::EnumIsScoped).add({
            {   11, PkceMethod::S256 },
            {   12, PkceMethod::Plain },
            {   13, PkceMethod::None },
        }),
    };
    QtMocHelpers::UintData qt_constructors {};
    QtMocHelpers::ClassInfos qt_classinfo({
            {    1,    2 },
    });
    return QtMocHelpers::metaObjectData<QOAuth2AuthorizationCodeFlow, qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums, qt_constructors, qt_classinfo);
}
Q_CONSTINIT const QMetaObject QOAuth2AuthorizationCodeFlow::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractOAuth2::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>.metaTypes,
    nullptr
} };

void QOAuth2AuthorizationCodeFlow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QOAuth2AuthorizationCodeFlow *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->accessTokenUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 1: _t->grant(); break;
        case 2: _t->refreshAccessToken(); break;
        case 3: _t->refreshTokensImplementation(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QOAuth2AuthorizationCodeFlow::*)(const QUrl & )>(_a, &QOAuth2AuthorizationCodeFlow::accessTokenUrlChanged, 0))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QUrl*>(_v) = _t->accessTokenUrl(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setAccessTokenUrl(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QOAuth2AuthorizationCodeFlow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QOAuth2AuthorizationCodeFlow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN28QOAuth2AuthorizationCodeFlowE_t>.strings))
        return static_cast<void*>(this);
    return QAbstractOAuth2::qt_metacast(_clname);
}

int QOAuth2AuthorizationCodeFlow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractOAuth2::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 4)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 4;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 4)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 4;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 1;
    }
    return _id;
}

// SIGNAL 0
void QOAuth2AuthorizationCodeFlow::accessTokenUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}
QT_WARNING_POP
