/****************************************************************************
** Meta object code from reading C++ file 'qoauth1.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtnetworkauth/src/oauth/qoauth1.h"
#include <QtNetwork/QSslError>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qoauth1.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7QOAuth1E_t {};
} // unnamed namespace

template <> constexpr inline auto QOAuth1::qt_create_metaobjectdata<qt_meta_tag_ZN7QOAuth1E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QOAuth1",
        "signatureMethodChanged",
        "",
        "QOAuth1::SignatureMethod",
        "method",
        "clientSharedSecretChanged",
        "credential",
        "tokenSecretChanged",
        "token",
        "temporaryCredentialsUrlChanged",
        "QUrl",
        "url",
        "tokenCredentialsUrlChanged",
        "grant",
        "continueGrantWithVerifier",
        "verifier",
        "SignatureMethod",
        "Hmac_Sha1",
        "Rsa_Sha1",
        "PlainText"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'signatureMethodChanged'
        QtMocHelpers::SignalData<void(QOAuth1::SignatureMethod)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'clientSharedSecretChanged'
        QtMocHelpers::SignalData<void(const QString &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 6 },
        }}),
        // Signal 'tokenSecretChanged'
        QtMocHelpers::SignalData<void(const QString &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 8 },
        }}),
        // Signal 'temporaryCredentialsUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'tokenCredentialsUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'grant'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'continueGrantWithVerifier'
        QtMocHelpers::SlotData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'SignatureMethod'
        QtMocHelpers::EnumData<enum SignatureMethod>(16, 16, QMC::EnumIsScoped).add({
            {   17, SignatureMethod::Hmac_Sha1 },
            {   18, SignatureMethod::Rsa_Sha1 },
            {   19, SignatureMethod::PlainText },
        }),
    };
    return QtMocHelpers::metaObjectData<QOAuth1, qt_meta_tag_ZN7QOAuth1E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QOAuth1::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractOAuth::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QOAuth1E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QOAuth1E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7QOAuth1E_t>.metaTypes,
    nullptr
} };

void QOAuth1::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QOAuth1 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->signatureMethodChanged((*reinterpret_cast<std::add_pointer_t<QOAuth1::SignatureMethod>>(_a[1]))); break;
        case 1: _t->clientSharedSecretChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->tokenSecretChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 3: _t->temporaryCredentialsUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->tokenCredentialsUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 5: _t->grant(); break;
        case 6: _t->continueGrantWithVerifier((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QOAuth1::*)(QOAuth1::SignatureMethod )>(_a, &QOAuth1::signatureMethodChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth1::*)(const QString & )>(_a, &QOAuth1::clientSharedSecretChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth1::*)(const QString & )>(_a, &QOAuth1::tokenSecretChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth1::*)(const QUrl & )>(_a, &QOAuth1::temporaryCredentialsUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QOAuth1::*)(const QUrl & )>(_a, &QOAuth1::tokenCredentialsUrlChanged, 4))
            return;
    }
}

const QMetaObject *QOAuth1::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QOAuth1::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7QOAuth1E_t>.strings))
        return static_cast<void*>(this);
    return QAbstractOAuth::qt_metacast(_clname);
}

int QOAuth1::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractOAuth::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 7;
    }
    return _id;
}

// SIGNAL 0
void QOAuth1::signatureMethodChanged(QOAuth1::SignatureMethod _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QOAuth1::clientSharedSecretChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QOAuth1::tokenSecretChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QOAuth1::temporaryCredentialsUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QOAuth1::tokenCredentialsUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}
QT_WARNING_POP
