/****************************************************************************
** Meta object code from reading C++ file 'qabstractoauth2.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtnetworkauth/src/oauth/qabstractoauth2.h"
#include <QtCore/qmetatype.h>
#include <QtCore/QSet>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractoauth2.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN15QAbstractOAuth2E_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractOAuth2::qt_create_metaobjectdata<qt_meta_tag_ZN15QAbstractOAuth2E_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractOAuth2",
        "scopeChanged",
        "",
        "scope",
        "grantedScopeTokensChanged",
        "QSet<QByteArray>",
        "tokens",
        "requestedScopeTokensChanged",
        "userAgentChanged",
        "userAgent",
        "responseTypeChanged",
        "responseType",
        "clientIdentifierSharedKeyChanged",
        "clientIdentifierSharedKey",
        "stateChanged",
        "state",
        "expirationAtChanged",
        "expiration",
        "refreshTokenChanged",
        "refreshToken",
        "accessTokenAboutToExpire",
        "refreshLeadTimeChanged",
        "std::chrono::seconds",
        "leadTime",
        "autoRefreshChanged",
        "enable",
        "nonceModeChanged",
        "NonceMode",
        "mode",
        "nonceChanged",
        "nonce",
        "idTokenChanged",
        "idToken",
        "tokenUrlChanged",
        "QUrl",
        "tokenUrl",
        "sslConfigurationChanged",
        "QSslConfiguration",
        "configuration",
        "error",
        "errorDescription",
        "uri",
        "serverReportedErrorOccurred",
        "authorizationCallbackReceived",
        "QVariantMap",
        "data",
        "refreshTokensImplementation",
        "refreshTokens",
        "createAuthenticatedUrl",
        "url",
        "parameters",
        "head",
        "QNetworkReply*",
        "get",
        "post",
        "QHttpMultiPart*",
        "multiPart",
        "put",
        "deleteResource",
        "grantedScopeTokens",
        "requestedScopeTokens",
        "refreshLeadTime",
        "autoRefresh",
        "nonceMode",
        "Automatic",
        "Enabled",
        "Disabled"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'scopeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic | QMC::MethodCompatibility, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'grantedScopeTokensChanged'
        QtMocHelpers::SignalData<void(const QSet<QByteArray> &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'requestedScopeTokensChanged'
        QtMocHelpers::SignalData<void(const QSet<QByteArray> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'userAgentChanged'
        QtMocHelpers::SignalData<void(const QString &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 9 },
        }}),
        // Signal 'responseTypeChanged'
        QtMocHelpers::SignalData<void(const QString &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 11 },
        }}),
        // Signal 'clientIdentifierSharedKeyChanged'
        QtMocHelpers::SignalData<void(const QString &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 13 },
        }}),
        // Signal 'stateChanged'
        QtMocHelpers::SignalData<void(const QString &)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 15 },
        }}),
        // Signal 'expirationAtChanged'
        QtMocHelpers::SignalData<void(const QDateTime &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QDateTime, 17 },
        }}),
        // Signal 'refreshTokenChanged'
        QtMocHelpers::SignalData<void(const QString &)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 19 },
        }}),
        // Signal 'accessTokenAboutToExpire'
        QtMocHelpers::SignalData<void()>(20, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'refreshLeadTimeChanged'
        QtMocHelpers::SignalData<void(std::chrono::seconds)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 22, 23 },
        }}),
        // Signal 'autoRefreshChanged'
        QtMocHelpers::SignalData<void(bool)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 25 },
        }}),
        // Signal 'nonceModeChanged'
        QtMocHelpers::SignalData<void(enum NonceMode)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 27, 28 },
        }}),
        // Signal 'nonceChanged'
        QtMocHelpers::SignalData<void(const QString &)>(29, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 30 },
        }}),
        // Signal 'idTokenChanged'
        QtMocHelpers::SignalData<void(const QString &)>(31, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 32 },
        }}),
        // Signal 'tokenUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(33, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 34, 35 },
        }}),
        // Signal 'sslConfigurationChanged'
        QtMocHelpers::SignalData<void(const QSslConfiguration &)>(36, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 37, 38 },
        }}),
        // Signal 'error'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QUrl &)>(39, 2, QMC::AccessPublic | QMC::MethodCompatibility, QMetaType::Void, {{
            { QMetaType::QString, 39 }, { QMetaType::QString, 40 }, { 0x80000000 | 34, 41 },
        }}),
        // Signal 'serverReportedErrorOccurred'
        QtMocHelpers::SignalData<void(const QString &, const QString &, const QUrl &)>(42, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 39 }, { QMetaType::QString, 40 }, { 0x80000000 | 34, 41 },
        }}),
        // Signal 'authorizationCallbackReceived'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(43, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'refreshTokensImplementation'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'refreshTokens'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'createAuthenticatedUrl'
        QtMocHelpers::MethodData<QUrl(const QUrl &, const QVariantMap &)>(48, 2, QMC::AccessPublic, 0x80000000 | 34, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'createAuthenticatedUrl'
        QtMocHelpers::MethodData<QUrl(const QUrl &)>(48, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 34, {{
            { 0x80000000 | 34, 49 },
        }}),
        // Method 'head'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(51, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'head'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(51, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 },
        }}),
        // Method 'get'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(53, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'get'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(53, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(54, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(54, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QByteArray &)>(54, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { QMetaType::QByteArray, 45 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, QHttpMultiPart *)>(54, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 55, 56 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(57, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(57, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QByteArray &)>(57, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { QMetaType::QByteArray, 45 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, QHttpMultiPart *)>(57, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 55, 56 },
        }}),
        // Method 'deleteResource'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(58, 2, QMC::AccessPublic, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 }, { 0x80000000 | 44, 50 },
        }}),
        // Method 'deleteResource'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(58, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 52, {{
            { 0x80000000 | 34, 49 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'scope'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'grantedScopeTokens'
        QtMocHelpers::PropertyData<QSet<QByteArray>>(59, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 1),
        // property 'requestedScopeTokens'
        QtMocHelpers::PropertyData<QSet<QByteArray>>(60, 0x80000000 | 5, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 2),
        // property 'userAgent'
        QtMocHelpers::PropertyData<QString>(9, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 3),
        // property 'clientIdentifierSharedKey'
        QtMocHelpers::PropertyData<QString>(13, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 5),
        // property 'state'
        QtMocHelpers::PropertyData<QString>(15, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 6),
        // property 'expiration'
        QtMocHelpers::PropertyData<QDateTime>(17, QMetaType::QDateTime, QMC::DefaultPropertyFlags, 7),
        // property 'refreshToken'
        QtMocHelpers::PropertyData<QString>(19, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 8),
        // property 'refreshLeadTime'
        QtMocHelpers::PropertyData<std::chrono::seconds>(61, 0x80000000 | 22, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 10),
        // property 'autoRefresh'
        QtMocHelpers::PropertyData<bool>(62, QMetaType::Bool, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 11),
        // property 'nonceMode'
        QtMocHelpers::PropertyData<enum NonceMode>(63, 0x80000000 | 27, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 12),
        // property 'nonce'
        QtMocHelpers::PropertyData<QString>(30, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 13),
        // property 'idToken'
        QtMocHelpers::PropertyData<QString>(32, QMetaType::QString, QMC::DefaultPropertyFlags, 14),
        // property 'tokenUrl'
        QtMocHelpers::PropertyData<QUrl>(35, 0x80000000 | 34, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 15),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'NonceMode'
        QtMocHelpers::EnumData<enum NonceMode>(27, 27, QMC::EnumIsScoped).add({
            {   64, NonceMode::Automatic },
            {   65, NonceMode::Enabled },
            {   66, NonceMode::Disabled },
        }),
    };
    return QtMocHelpers::metaObjectData<QAbstractOAuth2, qt_meta_tag_ZN15QAbstractOAuth2E_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstractOAuth2::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractOAuth::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractOAuth2E_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractOAuth2E_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN15QAbstractOAuth2E_t>.metaTypes,
    nullptr
} };

void QAbstractOAuth2::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractOAuth2 *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->scopeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->grantedScopeTokensChanged((*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[1]))); break;
        case 2: _t->requestedScopeTokensChanged((*reinterpret_cast<std::add_pointer_t<QSet<QByteArray>>>(_a[1]))); break;
        case 3: _t->userAgentChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 4: _t->responseTypeChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 5: _t->clientIdentifierSharedKeyChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 6: _t->stateChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 7: _t->expirationAtChanged((*reinterpret_cast<std::add_pointer_t<QDateTime>>(_a[1]))); break;
        case 8: _t->refreshTokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 9: _t->accessTokenAboutToExpire(); break;
        case 10: _t->refreshLeadTimeChanged((*reinterpret_cast<std::add_pointer_t<std::chrono::seconds>>(_a[1]))); break;
        case 11: _t->autoRefreshChanged((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 12: _t->nonceModeChanged((*reinterpret_cast<std::add_pointer_t<enum NonceMode>>(_a[1]))); break;
        case 13: _t->nonceChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 14: _t->idTokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 15: _t->tokenUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 16: _t->sslConfigurationChanged((*reinterpret_cast<std::add_pointer_t<QSslConfiguration>>(_a[1]))); break;
        case 17: _t->error((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 18: _t->serverReportedErrorOccurred((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[3]))); break;
        case 19: _t->authorizationCallbackReceived((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 20: _t->refreshTokensImplementation(); break;
        case 21: _t->refreshTokens(); break;
        case 22: { QUrl _r = _t->createAuthenticatedUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 23: { QUrl _r = _t->createAuthenticatedUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QUrl*>(_a[0]) = std::move(_r); }  break;
        case 24: { QNetworkReply* _r = _t->head((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 25: { QNetworkReply* _r = _t->head((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 26: { QNetworkReply* _r = _t->get((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 27: { QNetworkReply* _r = _t->get((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 28: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 29: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 30: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 31: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QHttpMultiPart*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 32: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 33: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 34: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 35: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QHttpMultiPart*>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 36: { QNetworkReply* _r = _t->deleteResource((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 37: { QNetworkReply* _r = _t->deleteResource((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QSet<QByteArray> >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::scopeChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QSet<QByteArray> & )>(_a, &QAbstractOAuth2::grantedScopeTokensChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QSet<QByteArray> & )>(_a, &QAbstractOAuth2::requestedScopeTokensChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::userAgentChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::responseTypeChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::clientIdentifierSharedKeyChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::stateChanged, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QDateTime & )>(_a, &QAbstractOAuth2::expirationAtChanged, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::refreshTokenChanged, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)()>(_a, &QAbstractOAuth2::accessTokenAboutToExpire, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(std::chrono::seconds )>(_a, &QAbstractOAuth2::refreshLeadTimeChanged, 10))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(bool )>(_a, &QAbstractOAuth2::autoRefreshChanged, 11))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(NonceMode )>(_a, &QAbstractOAuth2::nonceModeChanged, 12))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::nonceChanged, 13))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & )>(_a, &QAbstractOAuth2::idTokenChanged, 14))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QUrl & )>(_a, &QAbstractOAuth2::tokenUrlChanged, 15))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QSslConfiguration & )>(_a, &QAbstractOAuth2::sslConfigurationChanged, 16))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & , const QString & , const QUrl & )>(_a, &QAbstractOAuth2::error, 17))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QString & , const QString & , const QUrl & )>(_a, &QAbstractOAuth2::serverReportedErrorOccurred, 18))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth2::*)(const QVariantMap & )>(_a, &QAbstractOAuth2::authorizationCallbackReceived, 19))
            return;
    }
    if (_c == QMetaObject::RegisterPropertyMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
        case 1:
            *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QSet<QByteArray> >(); break;
        }
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->scope(); break;
        case 1: *reinterpret_cast<QSet<QByteArray>*>(_v) = _t->grantedScopeTokens(); break;
        case 2: *reinterpret_cast<QSet<QByteArray>*>(_v) = _t->requestedScopeTokens(); break;
        case 3: *reinterpret_cast<QString*>(_v) = _t->userAgent(); break;
        case 4: *reinterpret_cast<QString*>(_v) = _t->clientIdentifierSharedKey(); break;
        case 5: *reinterpret_cast<QString*>(_v) = _t->state(); break;
        case 6: *reinterpret_cast<QDateTime*>(_v) = _t->expirationAt(); break;
        case 7: *reinterpret_cast<QString*>(_v) = _t->refreshToken(); break;
        case 8: *reinterpret_cast<std::chrono::seconds*>(_v) = _t->refreshLeadTime(); break;
        case 9: *reinterpret_cast<bool*>(_v) = _t->autoRefresh(); break;
        case 10: *reinterpret_cast<enum NonceMode*>(_v) = _t->nonceMode(); break;
        case 11: *reinterpret_cast<QString*>(_v) = _t->nonce(); break;
        case 12: *reinterpret_cast<QString*>(_v) = _t->idToken(); break;
        case 13: *reinterpret_cast<QUrl*>(_v) = _t->tokenUrl(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setScope(*reinterpret_cast<QString*>(_v)); break;
        case 2: _t->setRequestedScopeTokens(*reinterpret_cast<QSet<QByteArray>*>(_v)); break;
        case 3: _t->setUserAgent(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setClientIdentifierSharedKey(*reinterpret_cast<QString*>(_v)); break;
        case 5: _t->setState(*reinterpret_cast<QString*>(_v)); break;
        case 7: _t->setRefreshToken(*reinterpret_cast<QString*>(_v)); break;
        case 8: _t->setRefreshLeadTime(*reinterpret_cast<std::chrono::seconds*>(_v)); break;
        case 9: _t->setAutoRefresh(*reinterpret_cast<bool*>(_v)); break;
        case 10: _t->setNonceMode(*reinterpret_cast<enum NonceMode*>(_v)); break;
        case 11: _t->setNonce(*reinterpret_cast<QString*>(_v)); break;
        case 13: _t->setTokenUrl(*reinterpret_cast<QUrl*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractOAuth2::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractOAuth2::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN15QAbstractOAuth2E_t>.strings))
        return static_cast<void*>(this);
    return QAbstractOAuth::qt_metacast(_clname);
}

int QAbstractOAuth2::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractOAuth::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 38)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 38;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void QAbstractOAuth2::scopeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstractOAuth2::grantedScopeTokensChanged(const QSet<QByteArray> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstractOAuth2::requestedScopeTokensChanged(const QSet<QByteArray> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstractOAuth2::userAgentChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstractOAuth2::responseTypeChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstractOAuth2::clientIdentifierSharedKeyChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstractOAuth2::stateChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstractOAuth2::expirationAtChanged(const QDateTime & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstractOAuth2::refreshTokenChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1);
}

// SIGNAL 9
void QAbstractOAuth2::accessTokenAboutToExpire()
{
    QMetaObject::activate(this, &staticMetaObject, 9, nullptr);
}

// SIGNAL 10
void QAbstractOAuth2::refreshLeadTimeChanged(std::chrono::seconds _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}

// SIGNAL 11
void QAbstractOAuth2::autoRefreshChanged(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 11, nullptr, _t1);
}

// SIGNAL 12
void QAbstractOAuth2::nonceModeChanged(NonceMode _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 12, nullptr, _t1);
}

// SIGNAL 13
void QAbstractOAuth2::nonceChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 13, nullptr, _t1);
}

// SIGNAL 14
void QAbstractOAuth2::idTokenChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 14, nullptr, _t1);
}

// SIGNAL 15
void QAbstractOAuth2::tokenUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 15, nullptr, _t1);
}

// SIGNAL 16
void QAbstractOAuth2::sslConfigurationChanged(const QSslConfiguration & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 16, nullptr, _t1);
}

// SIGNAL 17
void QAbstractOAuth2::error(const QString & _t1, const QString & _t2, const QUrl & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 17, nullptr, _t1, _t2, _t3);
}

// SIGNAL 18
void QAbstractOAuth2::serverReportedErrorOccurred(const QString & _t1, const QString & _t2, const QUrl & _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 18, nullptr, _t1, _t2, _t3);
}

// SIGNAL 19
void QAbstractOAuth2::authorizationCallbackReceived(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 19, nullptr, _t1);
}
QT_WARNING_POP
