/****************************************************************************
** Meta object code from reading C++ file 'qabstractoauth.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.12.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../qtnetworkauth/src/oauth/qabstractoauth.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'qabstractoauth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.12.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN14QAbstractOAuthE_t {};
} // unnamed namespace

template <> constexpr inline auto QAbstractOAuth::qt_create_metaobjectdata<qt_meta_tag_ZN14QAbstractOAuthE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "QAbstractOAuth",
        "clientIdentifierChanged",
        "",
        "clientIdentifier",
        "tokenChanged",
        "token",
        "statusChanged",
        "Status",
        "status",
        "authorizationUrlChanged",
        "QUrl",
        "url",
        "extraTokensChanged",
        "QVariantMap",
        "tokens",
        "contentTypeChanged",
        "ContentType",
        "contentType",
        "requestFailed",
        "Error",
        "error",
        "authorizeWithBrowser",
        "granted",
        "finished",
        "QNetworkReply*",
        "reply",
        "replyDataReceived",
        "data",
        "grant",
        "head",
        "parameters",
        "get",
        "post",
        "put",
        "deleteResource",
        "extraTokens",
        "authorizationUrl",
        "QAbstractOAuth::ContentType",
        "NotAuthenticated",
        "TemporaryCredentialsReceived",
        "Granted",
        "RefreshingToken",
        "Stage",
        "RequestingTemporaryCredentials",
        "RequestingAuthorization",
        "RequestingAccessToken",
        "RefreshingAccessToken",
        "NoError",
        "NetworkError",
        "ServerError",
        "OAuthTokenNotFoundError",
        "OAuthTokenSecretNotFoundError",
        "OAuthCallbackNotVerified",
        "ClientError",
        "ExpiredError"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'clientIdentifierChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'tokenChanged'
        QtMocHelpers::SignalData<void(const QString &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 5 },
        }}),
        // Signal 'statusChanged'
        QtMocHelpers::SignalData<void(enum Status)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 7, 8 },
        }}),
        // Signal 'authorizationUrlChanged'
        QtMocHelpers::SignalData<void(const QUrl &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'extraTokensChanged'
        QtMocHelpers::SignalData<void(const QVariantMap &)>(12, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 13, 14 },
        }}),
        // Signal 'contentTypeChanged'
        QtMocHelpers::SignalData<void(enum ContentType)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 16, 17 },
        }}),
        // Signal 'requestFailed'
        QtMocHelpers::SignalData<void(const Error)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 19, 20 },
        }}),
        // Signal 'authorizeWithBrowser'
        QtMocHelpers::SignalData<void(const QUrl &)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'granted'
        QtMocHelpers::SignalData<void()>(22, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'finished'
        QtMocHelpers::SignalData<void(QNetworkReply *)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 24, 25 },
        }}),
        // Signal 'replyDataReceived'
        QtMocHelpers::SignalData<void(const QByteArray &)>(26, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QByteArray, 27 },
        }}),
        // Slot 'grant'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessPublic, QMetaType::Void),
        // Method 'head'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(29, 2, QMC::AccessPublic, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 13, 30 },
        }}),
        // Method 'head'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(29, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'get'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(31, 2, QMC::AccessPublic, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 13, 30 },
        }}),
        // Method 'get'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(31, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(32, 2, QMC::AccessPublic, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 13, 30 },
        }}),
        // Method 'post'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(32, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(33, 2, QMC::AccessPublic, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 13, 30 },
        }}),
        // Method 'put'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(33, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Method 'deleteResource'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &, const QVariantMap &)>(34, 2, QMC::AccessPublic, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 }, { 0x80000000 | 13, 30 },
        }}),
        // Method 'deleteResource'
        QtMocHelpers::MethodData<QNetworkReply *(const QUrl &)>(34, 2, QMC::AccessPublic | QMC::MethodCloned, 0x80000000 | 24, {{
            { 0x80000000 | 10, 11 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
        // property 'clientIdentifier'
        QtMocHelpers::PropertyData<QString>(3, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 0),
        // property 'token'
        QtMocHelpers::PropertyData<QString>(5, QMetaType::QString, QMC::DefaultPropertyFlags | QMC::Writable | QMC::StdCppSet, 1),
        // property 'status'
        QtMocHelpers::PropertyData<enum Status>(8, 0x80000000 | 7, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 2),
        // property 'extraTokens'
        QtMocHelpers::PropertyData<QVariantMap>(35, 0x80000000 | 13, QMC::DefaultPropertyFlags | QMC::EnumOrFlag, 4),
        // property 'authorizationUrl'
        QtMocHelpers::PropertyData<QUrl>(36, 0x80000000 | 10, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 3),
        // property 'contentType'
        QtMocHelpers::PropertyData<QAbstractOAuth::ContentType>(17, 0x80000000 | 37, QMC::DefaultPropertyFlags | QMC::Writable | QMC::EnumOrFlag | QMC::StdCppSet, 5),
    };
    QtMocHelpers::UintData qt_enums {
        // enum 'Status'
        QtMocHelpers::EnumData<enum Status>(7, 7, QMC::EnumIsScoped).add({
            {   38, Status::NotAuthenticated },
            {   39, Status::TemporaryCredentialsReceived },
            {   40, Status::Granted },
            {   41, Status::RefreshingToken },
        }),
        // enum 'Stage'
        QtMocHelpers::EnumData<enum Stage>(42, 42, QMC::EnumIsScoped).add({
            {   43, Stage::RequestingTemporaryCredentials },
            {   44, Stage::RequestingAuthorization },
            {   45, Stage::RequestingAccessToken },
            {   46, Stage::RefreshingAccessToken },
        }),
        // enum 'Error'
        QtMocHelpers::EnumData<enum Error>(19, 19, QMC::EnumIsScoped).add({
            {   47, Error::NoError },
            {   48, Error::NetworkError },
            {   49, Error::ServerError },
            {   50, Error::OAuthTokenNotFoundError },
            {   51, Error::OAuthTokenSecretNotFoundError },
            {   52, Error::OAuthCallbackNotVerified },
            {   53, Error::ClientError },
            {   54, Error::ExpiredError },
        }),
    };
    return QtMocHelpers::metaObjectData<QAbstractOAuth, qt_meta_tag_ZN14QAbstractOAuthE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject QAbstractOAuth::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QAbstractOAuthE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QAbstractOAuthE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN14QAbstractOAuthE_t>.metaTypes,
    nullptr
} };

void QAbstractOAuth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<QAbstractOAuth *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->clientIdentifierChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->tokenChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 2: _t->statusChanged((*reinterpret_cast<std::add_pointer_t<enum Status>>(_a[1]))); break;
        case 3: _t->authorizationUrlChanged((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 4: _t->extraTokensChanged((*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[1]))); break;
        case 5: _t->contentTypeChanged((*reinterpret_cast<std::add_pointer_t<enum ContentType>>(_a[1]))); break;
        case 6: _t->requestFailed((*reinterpret_cast<std::add_pointer_t<enum Error>>(_a[1]))); break;
        case 7: _t->authorizeWithBrowser((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 8: _t->granted(); break;
        case 9: _t->finished((*reinterpret_cast<std::add_pointer_t<QNetworkReply*>>(_a[1]))); break;
        case 10: _t->replyDataReceived((*reinterpret_cast<std::add_pointer_t<QByteArray>>(_a[1]))); break;
        case 11: _t->grant(); break;
        case 12: { QNetworkReply* _r = _t->head((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 13: { QNetworkReply* _r = _t->head((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 14: { QNetworkReply* _r = _t->get((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 15: { QNetworkReply* _r = _t->get((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 16: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 17: { QNetworkReply* _r = _t->post((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 18: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 19: { QNetworkReply* _r = _t->put((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 20: { QNetworkReply* _r = _t->deleteResource((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QVariantMap>>(_a[2])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        case 21: { QNetworkReply* _r = _t->deleteResource((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1])));
            if (_a[0]) *reinterpret_cast<QNetworkReply**>(_a[0]) = std::move(_r); }  break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QString & )>(_a, &QAbstractOAuth::clientIdentifierChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QString & )>(_a, &QAbstractOAuth::tokenChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(Status )>(_a, &QAbstractOAuth::statusChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QUrl & )>(_a, &QAbstractOAuth::authorizationUrlChanged, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QVariantMap & )>(_a, &QAbstractOAuth::extraTokensChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(ContentType )>(_a, &QAbstractOAuth::contentTypeChanged, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const Error )>(_a, &QAbstractOAuth::requestFailed, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QUrl & )>(_a, &QAbstractOAuth::authorizeWithBrowser, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)()>(_a, &QAbstractOAuth::granted, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(QNetworkReply * )>(_a, &QAbstractOAuth::finished, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (QAbstractOAuth::*)(const QByteArray & )>(_a, &QAbstractOAuth::replyDataReceived, 10))
            return;
    }
    if (_c == QMetaObject::ReadProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast<QString*>(_v) = _t->clientIdentifier(); break;
        case 1: *reinterpret_cast<QString*>(_v) = _t->token(); break;
        case 2: *reinterpret_cast<enum Status*>(_v) = _t->status(); break;
        case 3: *reinterpret_cast<QVariantMap*>(_v) = _t->extraTokens(); break;
        case 4: *reinterpret_cast<QUrl*>(_v) = _t->authorizationUrl(); break;
        case 5: *reinterpret_cast<QAbstractOAuth::ContentType*>(_v) = _t->contentType(); break;
        default: break;
        }
    }
    if (_c == QMetaObject::WriteProperty) {
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setClientIdentifier(*reinterpret_cast<QString*>(_v)); break;
        case 1: _t->setToken(*reinterpret_cast<QString*>(_v)); break;
        case 4: _t->setAuthorizationUrl(*reinterpret_cast<QUrl*>(_v)); break;
        case 5: _t->setContentType(*reinterpret_cast<QAbstractOAuth::ContentType*>(_v)); break;
        default: break;
        }
    }
}

const QMetaObject *QAbstractOAuth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *QAbstractOAuth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN14QAbstractOAuthE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int QAbstractOAuth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 22)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 22;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 22)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 22;
    }
    if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::BindableProperty
            || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void QAbstractOAuth::clientIdentifierChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void QAbstractOAuth::tokenChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void QAbstractOAuth::statusChanged(Status _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void QAbstractOAuth::authorizationUrlChanged(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}

// SIGNAL 4
void QAbstractOAuth::extraTokensChanged(const QVariantMap & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void QAbstractOAuth::contentTypeChanged(ContentType _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void QAbstractOAuth::requestFailed(const Error _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1);
}

// SIGNAL 7
void QAbstractOAuth::authorizeWithBrowser(const QUrl & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 7, nullptr, _t1);
}

// SIGNAL 8
void QAbstractOAuth::granted()
{
    QMetaObject::activate(this, &staticMetaObject, 8, nullptr);
}

// SIGNAL 9
void QAbstractOAuth::finished(QNetworkReply * _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1);
}

// SIGNAL 10
void QAbstractOAuth::replyDataReceived(const QByteArray & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 10, nullptr, _t1);
}
QT_WARNING_POP
